#!/usr/bin/env python

import sys
try:
	 import pygtk
	 pygtk.require( "2.0" )
except:
	  pass
try:
	import gtk
	import gtk.glade
	import pango
	import os
	import commands
	import gconf
	import gnomeapplet
	import gobject
	import gettext
	import gnomevfs
	import traceback
except Exception, e:
	print e
	sys.exit( 1 )

sys.path.append( os.path.join(  os.path.abspath( os.path.dirname( sys.argv[0] ) ), "plugins") )

from easybuttons import iconManager
from easygconf import EasyGConf

gtk.glade.textdomain("mintMenu")
gtk.glade.bindtextdomain("mintMenu", "locale/")
gettext.textdomain("mintMenu")
gettext.bindtextdomain("mintMenu", "locale/")
_ = gettext.gettext



class MainWindow(object):
	"""This is the main class for the application"""

	def __init__( self, toggleButton ):

		self.path = os.path.abspath( os.path.dirname( sys.argv[0] ) )
		sys.path.append( os.path.join( self.path, "plugins") )

		self.toggle = toggleButton
		#Set the Glade file
		self.gladefile = os.path.join( self.path, "mintMenu_frontend.glade" )
		self.wTree = gtk.glade.XML( self.gladefile, "window1" )

		plugindir = os.path.join( os.path.expanduser( "~" ), ".linuxmint/mintMenu/plugins" )
		sys.path.append( plugindir )

		self.mintMenu = gtk.Tooltips()

		dic = {"on_window1_destroy" : gtk.main_quit,
		       "on_pin_button_toggled" : self.PinMenu}
		self.wTree.signal_autoconnect( dic )

		self.gconf = EasyGConf( "/apps/mintMenu/" )

		self.GetSetGconfEntries()
		self.SetupMintMenuBorder()
		self.PopulatePlugins()
		self.gconf.notifyAdd( "plugins_list", self.RegenPlugins )
		self.gconf.notifyAdd( "show_side_pane", self.SetupMintMenuBorder )


	def GetSetGconfEntries( self ):
		self.pluginlist = self.gconf.get( "list-string", "plugins_list",['newpane','places', 'system_management', 'newpane', 'applications'] )

		self.dottedfile = os.path.join( self.path, "dotted.png")
		self.usecustomcolor = self.gconf.get( "bool", "use_custom_color", True )
		self.customcolor = self.gconf.get( "color", "custom_color", "#FFFFFF" )
		self.customheadingcolor = self.gconf.get( "color", "custom_heading_color", "#005555" )
		self.custombordercolor = self.gconf.get( "color", "custom_border_color", "#005555" )
		self.borderwidth = self.gconf.get( "int", "border_width", 1 )
		self.sidepanevisible = self.gconf.get( "bool", "show_side_pane", False )
		self.offset = self.gconf.get( "int", "mintMenu_offset", -3 )
		self.pinmenu = self.gconf.get( "bool", "pin_menu", False )


	def PinMenu(self, *args, **kargs):
		if self.wTree.get_widget("pin_button").get_active() == True:
			self.gconf.set("bool", "pin_menu", True)
			self.pinmenu = True
			self.wTree.get_widget( "sidepane" ).show()
		else:
			self.gconf.set("bool", "pin_menu", False)
			self.pinmenu = False


	def SetupMintMenuBorder( self, *args, **kargs ):
		self.wTree.get_widget( "window1" ).modify_bg( gtk.STATE_NORMAL, gtk.gdk.color_parse( self.custombordercolor ) )
		self.wTree.get_widget( "border" ).set_padding( self.borderwidth, self.borderwidth, self.borderwidth, self.borderwidth )
		self.SetPaneColors( [self.wTree.get_widget( "sidepane" )] )
		self.sidepanevisible =  self.gconf.get( "bool", "show_side_pane", False )
		self.pinmenu =  self.gconf.get( "bool", "pin_menu", False )

		if self.pinmenu == False:
			self.wTree.get_widget( "pin_button" ).set_active(False)
		else:
			self.wTree.get_widget( "pin_button" ).set_active(True)

		if self.sidepanevisible == False and self.pinmenu == False:
			self.wTree.get_widget( "sidepane" ).hide()
		else:
			self.wTree.get_widget( "sidepane" ).show()


	def PopulatePlugins( self ):
		PluginPane = gtk.EventBox()
		PluginPane.show()
		PaneLadder = gtk.VBox( False, 0 )
		PluginPane.add( PaneLadder )
		self.SetPaneColors( [PluginPane] )
		ImageBox = gtk.EventBox()
		self.SetPaneColors( [ImageBox] )
		ImageBox.show()

		seperatorImage = gtk.gdk.pixbuf_new_from_file( self.dottedfile )
		
		paneholder = self.wTree.get_widget( "paneholder" )

		self.plugins = {}

		for plugin in self.pluginlist:
			if plugin in self.plugins:
				print "Duplicate plugin in list: ", plugin
				continue

			if plugin != "newpane":
				try:
					X = __import__( plugin )
					# If no parameter passed to plugin it is autonomous
					if X.pluginclass.__init__.func_code.co_argcount == 1:
						MyPlugin = X.pluginclass()
					else:
						# pass mintMenu and togglebutton instance so that the plugin can use it
						MyPlugin = X.pluginclass( self, self.toggle )

					if not MyPlugin.icon:
						MyPlugin.icon = "gnome-logo-icon.png"

					if hasattr( MyPlugin, "hideseparator" ) and not MyPlugin.hideseparator:
						Image1 = gtk.Image()
						Image1.set_from_pixbuf( seperatorImage )
						if not ImageBox.get_child():
							ImageBox.add( Image1 )
							Image1.show()
				
					print "Loading plugin '" + plugin + "' : sucessful"
				except Exception, e:
					MyPlugin = gtk.EventBox() #Fake class for MyPlugin
					MyPlugin.heading = "Couldn't load plugin \""+plugin+"\""
					MyPlugin.content_holder = gtk.EventBox()

					# create traceback
					info = sys.exc_info()

					errorLabel = gtk.Label( "\n".join(traceback.format_exception( info[0], info[1], info[2] )).replace("\\n", "\n") )
					errorLabel.set_selectable( True )
					errorLabel.set_line_wrap( True )
					errorLabel.set_alignment( 0.0, 0.0 )
					errorLabel.set_padding( 5, 5 )
					errorLabel.show()

					MyPlugin.content_holder.add( errorLabel )
					MyPlugin.add( MyPlugin.content_holder )
					MyPlugin.width = 270
					MyPlugin.icon = 'gnome-logo-icon.png'
					print "Unable to load " + plugin + " plugin :-("


				self.SetPaneColors( [MyPlugin.content_holder] )


				MyPlugin.content_holder.show()

				if MyPlugin.heading != "":
					VBox1 = gtk.VBox( False, 0 )
					Label1 = gtk.Label( MyPlugin.heading )
					Align1 = gtk.Alignment( 0, 0.5, 0, 0 )
					Align1.set_padding( 0, 0, 0, 0 )
					Align1.add( Label1 )
					self.SetHeadingStyle( [Label1] )
					Align1.show()
					Label1.show()

					if not hasattr( MyPlugin, 'sticky' ) or MyPlugin.sticky == True:
						heading = gtk.EventBox()
						Align1.set_padding( 0, 0, 10, 0 )
						self.SetPaneColors( [heading] )
						heading.set_size_request( MyPlugin.width, 30 )
					else:
						heading = gtk.Button( )
						heading.set_relief( gtk.RELIEF_NONE )
						heading.set_size_request( MyPlugin.width, -1 )
						heading.connect( "button_press_event", self.TogglePluginView, VBox1, MyPlugin.icon, MyPlugin.heading, MyPlugin )

					heading.add( Align1 )
					heading.show()
					VBox1.pack_start( heading, False )
					VBox1.show()
					MyPlugin.container = VBox1
					#Add plugin to Plugin Box under heading button
					MyPlugin.content_holder.reparent( VBox1 )

					#Add plugin to main window
					PaneLadder.pack_start( VBox1 )
					PaneLadder.show()

				if MyPlugin.window:
					MyPlugin.window.destroy()

				if hasattr( MyPlugin, 'do_plugin' ):
					MyPlugin.do_plugin()
				if hasattr( MyPlugin, 'height' ):
					MyPlugin.content_holder.set_size_request( -1, MyPlugin.height )
				if hasattr( MyPlugin, 'itemstocolor' ):
					self.SetPaneColors( MyPlugin.itemstocolor )
				if hasattr( MyPlugin, 'minimized' ):
					if MyPlugin.minimized == True:
						self.HidePlugin( MyPlugin.container, MyPlugin.icon, MyPlugin.heading, MyPlugin )

				self.plugins[plugin] = MyPlugin

			else:
				paneholder.pack_start( ImageBox, False, False, 0 )
				paneholder.pack_start( PluginPane, False, False, 0 )
				PluginPane = gtk.EventBox()
				PluginPane.show()
				PaneLadder = gtk.VBox( False, 0 )
				PluginPane.add( PaneLadder )
				self.SetPaneColors( [PluginPane] )
				ImageBox = gtk.EventBox()
				self.SetPaneColors( [ImageBox] )
				ImageBox.show()

				if self.plugins and hasattr( MyPlugin, 'hideseparator' ) and not MyPlugin.hideseparator: 
					Image1 = gtk.Image()
					Image1.set_from_pixbuf( seperatorImage )
					Image1.show()
					ImageBox.add( Image1 )


		paneholder.pack_start( ImageBox, False, False, 0 )
		paneholder.pack_start( PluginPane, False, False, 0 )


	def HidePlugin( self , PluginData, icon, tooltip, plugin ):
		if PluginData.flags() & gtk.VISIBLE:
			PluginData.hide()
			Button1 = gtk.Button( '', "ok", True )
			Button1.remove( Button1.get_children()[0] )
			Image1 = gtk.Image()
			Image1.set_from_pixbuf( iconManager.getIcon( icon, gtk.ICON_SIZE_MENU ) )
			Button1.add( Image1 )
			Button1.show()
			self.mintMenu.set_tip( Button1, tooltip, tip_private=None )
			Image1.show()

			#Show side pane if it is hidden
			self.wTree.get_widget( "sidepane" ).show()
			Button1.connect( "button_press_event", self.TogglePluginView, PluginData, icon, tooltip, plugin )
			self.wTree.get_widget( "vbox26" ).pack_start( Button1, False, False, 0 )


	def TogglePluginView( self, widget, event, PluginData, icon, tooltip, plugin ):
		if event.button == 1:
			if PluginData.flags() & gtk.VISIBLE:
				PluginData.hide()
				if hasattr( plugin, 'SetHidden' ):
					plugin.SetHidden(True)
				Button1 = gtk.Button( '', "ok", True )
				Button1.remove( Button1.get_children()[0] )
				Image1 = gtk.Image()
				Image1.set_from_pixbuf( iconManager.getIcon( icon, gtk.ICON_SIZE_MENU ) )
				Button1.add( Image1 )
				Button1.show()
				self.mintMenu.set_tip( Button1, tooltip, tip_private=None )
				Image1.show()

				#Show side pane if it is hidden
				self.wTree.get_widget( "sidepane" ).show()
				Button1.connect( "button_press_event", self.TogglePluginView, PluginData, icon, tooltip, plugin )
				self.wTree.get_widget( "vbox26" ).pack_start( Button1, False, False, 0 )
				
			else:
				PluginData.show()
				if hasattr( plugin, 'SetHidden' ):
					plugin.SetHidden(False)
				widget.destroy()
				if len( self.wTree.get_widget( "vbox26" ).get_children() ) == 0:
					if self.sidepanevisible == False and self.pinmenu == False:
						self.wTree.get_widget( "sidepane" ).hide()
					else:
						self.wTree.get_widget( "sidepane" ).show()

	def SetPaneColors( self, items ):
		if self.usecustomcolor:
			for item in items:
				item.modify_bg( gtk.STATE_NORMAL, gtk.gdk.color_parse( self.customcolor ) )

	def SetHeadingStyle( self, items ):
		HeadingStyle = pango.AttrList()
		attr = pango.AttrSize( 12000, 0, -1 )
		HeadingStyle.insert( attr )

		if self.usecustomcolor:
			headingcolor = gtk.gdk.color_parse( self.customheadingcolor )
			attr = pango.AttrForeground( headingcolor.red, headingcolor.green, headingcolor.blue, 0, -1 )
			HeadingStyle.insert( attr )

		attr = pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 )
		HeadingStyle.insert( attr )

		for item in items:
			item.set_attributes( HeadingStyle )


	def RegenPlugins( self, *args, **kargs ):
		print
		print "Reloading Plugins..."
		for items in self.wTree.get_widget( "paneholder" ):
			if items.name != "sidepane":
				items.destroy()
		self.GetSetGconfEntries()
		self.PopulatePlugins()
		self.SetupMintMenuBorder()
		print "mintMenu reloaded"

	def show( self ):
		# This is very, very weird, but there is something wrong
		# when using this applet in "metacity" that causes
		# some weird?? bug and we just have to call .present()
		# twice to solve it... found this out by trial and error...
		# profoX
		window1 = self.wTree.get_widget("window1")
		window1.present()
		window1.stick()
		window1.present()

		if ( "applications" in self.plugins ) and ( hasattr( self.plugins["applications"], "focusSearchEntry" ) ):
			self.plugins["applications"].focusSearchEntry()


	def hide( self ):
		if not self.pinmenu:
			self.wTree.get_widget("window1").hide()


class MenuWin:
	def __init__(self, applet, iid):
		self.applet = applet

		self.gconf = EasyGConf( "/apps/mintMenu/" )
		self.gconf.notifyAdd( "applet_text", self.GetGconfEntries )
		self.gconf.notifyAdd( "applet_icon_name", self.GetGconfEntries )
		self.gconf.notifyAdd( "hide_applet_icon", self.GetGconfEntries )
		self.gconf.notifyAdd( "applet_icon_size", self.GetGconfEntries )
		self.GetGconfEntries()

		self.toggle = gtk.ToggleButton()
		self.button_icon = gtk.image_new_from_icon_name("distributor-logo", gtk.ICON_SIZE_MENU)
		icon = iconManager.getIcon("distributor-logo", 1)
		if icon:
			gtk.window_set_default_icon( icon )
		self.orientation = self.applet.get_orient()
		# if we have a horizontal panel
		if self.orientation == gnomeapplet.ORIENT_UP or self.orientation == gnomeapplet.ORIENT_DOWN:
			self.button_box = gtk.HBox()
			self.systemlabel = gtk.Label( self.buttonText )
			self.button_box.set_homogeneous(False)
			self.button_box.pack_start(self.button_icon)
			self.button_box.pack_start(self.systemlabel)
		# if we have a vertical panel
		if self.orientation == gnomeapplet.ORIENT_LEFT:
			self.button_box = gtk.VBox()
			self.systemlabel = gtk.Label( self.buttonText )
			self.button_box.set_homogeneous(False)
			self.systemlabel.set_angle(90)
			self.button_box.pack_start(self.systemlabel)
			self.button_box.pack_start(self.button_icon)
		if self.orientation == gnomeapplet.ORIENT_RIGHT:
			self.button_box = gtk.VBox()
			self.systemlabel = gtk.Label( self.buttonText )
			self.button_box.set_homogeneous(False)
			self.systemlabel.set_angle(270)
			self.button_box.pack_start(self.button_icon)
			self.button_box.pack_start(self.systemlabel)


		self.toggle.add(self.button_box)
		self.toggle.set_relief(gtk.RELIEF_NONE)
		self.toggle.set_active(False)
		self.toggle.connect("enter-notify-event", self.enterNotify)
		self.toggle.connect("button-press-event", self.showMenu)
		self.applet.connect("change-orient", self.change_orientation)
		#self.applet.set_applet_flags(gnomeapplet.EXPAND_MINOR | gnomeapplet.EXPAND_MAJOR)
		# Change color
		self.applet.connect("change_background", self.change_background)
		self.applet.add( self.toggle )
		self.mainwin = MainWindow( self.toggle )
		self.mainwin.wTree.get_widget("window1").add_events( gtk.gdk.FOCUS_CHANGE )
		self.mainwin.wTree.get_widget("window1").connect( "focus_out_event", self.loseFocus )

		self.propxml="""
		        <popup name="button3">
			<menuitem name="Item 1" verb="Preferences" label="%s" pixtype="stock" pixname="gtk-preferences" />
			<menuitem name="Item 1" verb="Edit" label="%s" pixtype="stock" pixname="gtk-exec" />
		        <menuitem name="Item 2" verb="Reload" label="%s" pixtype="stock" pixname="gtk-refresh" />
		        <menuitem name="Item 3" verb="About" label="%s" pixtype="stock" pixname="gtk-about" />
		        </popup>
		        """ % ( _("_Preferences"), _("_Edit Menu"), _("_Reload Plugins"), _("_About") )
		self.verbs = [("Preferences", self.showPreferences),("Edit", self.showMenuEditor),("About", self.showAboutDialog),("Reload",self.mainwin.RegenPlugins)]


	def GetGconfEntries( self, *args, **kargs ):
		self.hideIcon =  self.gconf.get( "bool", "hide_applet_icon", False )
		self.buttonText =  self.gconf.get( "string", "applet_text", " Cassandra" )
		self.customIcon =  self.gconf.get( "string", "applet_icon_name", "distributor-logo" )
		self.customIconSize =  self.gconf.get( "int", "applet_icon_size", 1 )
		self.SetIconSize(self.customIconSize)

	def SetIconSize( self, icon_size):

		if icon_size == 4:
			icon_size = gtk.ICON_SIZE_DIALOG
		if icon_size == 3:
			icon_size = gtk.ICON_SIZE_DND
		if icon_size == 2:
			icon_size = gtk.ICON_SIZE_BUTTON
		if icon_size == 1:
			icon_size = gtk.ICON_SIZE_MENU

		self.IconSize = icon_size

	def change_background(self, applet, type, color, pixmap):
		# get reset style
		self.applet.set_style(None)
		rc_style = gtk.RcStyle()
		self.applet.modify_style(rc_style)

		if (gnomeapplet.COLOR_BACKGROUND == type):
			applet.modify_bg(gtk.STATE_NORMAL, color)
		elif (gnomeapplet.PIXMAP_BACKGROUND == type):
			style = applet.style
			style.bg_pixmap[gtk.STATE_NORMAL] = pixmap
			applet.set_style(style)


	def change_orientation(self, *args, **kargs):
		self.orientation = self.applet.get_orient()

		if self.orientation == gnomeapplet.ORIENT_UP or self.orientation == gnomeapplet.ORIENT_DOWN:
			tmpbox = gtk.HBox()
			self.systemlabel.set_angle(0)
			tmpbox.set_homogeneous(False)
			self.button_box.reorder_child(self.button_icon, 0)
		if self.orientation == gnomeapplet.ORIENT_LEFT:
			tmpbox = gtk.VBox()
			self.systemlabel.set_angle(90)
			tmpbox.set_homogeneous(False)
			self.button_box.reorder_child(self.button_icon, 1)
		if self.orientation == gnomeapplet.ORIENT_RIGHT:
			tmpbox = gtk.VBox()
			self.systemlabel.set_angle(270)
			tmpbox.set_homogeneous(False)
			self.button_box.reorder_child(self.button_icon, 0)

		# reparent all the hboxes to the new tmpbox
		for i in (self.button_box.get_children()):
			i.reparent(tmpbox)

		self.toggle.remove(self.button_box)
		self.button_box = tmpbox
		self.toggle.add(self.button_box)
		self.applet.show_all()

		# this call makes sure width stays intact
		self.changeButton(self.buttonText, self.hideIcon, self.customIcon)


	def enterNotify(self, *args, **kargs):
		return True

	def changeButton(self, text, hideIcon, customIcon):
		self.systemlabel.set_text(text)
		# This code calculates width and height for the button_box
		# and takes the orientation in account
		if customIcon != "":
			self.button_icon.set_from_icon_name(customIcon, self.IconSize)
		else:
			self.button_icon.set_from_icon_name("distributor-logo", self.IconSize)

		if self.orientation == gnomeapplet.ORIENT_UP or self.orientation == gnomeapplet.ORIENT_DOWN:
			if hideIcon == False:
				self.button_box.set_size_request(self.systemlabel.size_request()[0] + 25, -1)
				self.button_icon.show()
			else:
				self.button_box.set_size_request(self.systemlabel.size_request()[0] + 2, -1)
				self.button_icon.hide()
		else:
			if hideIcon == False:
				self.button_box.set_size_request(-1, self.systemlabel.size_request()[1] + 25)
				self.button_icon.show()
			else:
				self.button_box.set_size_request(-1, self.systemlabel.size_request()[1] + 2)
				self.button_icon.hide()



	def showAboutDialog(self, uicomponent, verb):

		gtk.about_dialog_set_email_hook( lambda dialog, mail: gnomevfs.url_show("mailto:" + mail) )
		gtk.about_dialog_set_url_hook( lambda dialog, url: gnomevfs.url_show(url) )
		about = gtk.AboutDialog()
		about.set_name( "mintMenu" )
		about.set_version( "2.2" )
		about.set_comments( _("a Slab-like Gnome Menu for Linux Mint") )
		about.set_authors( ["Clement Lefebvre <root@linuxmint.com>", "Lars-Peter Clausen <lars@laprican.de>"] )
		about.set_translator_credits( _("translator-credits") )
		about.set_website( "http://linuxmint.com" )
		about.set_copyright( _("Based on USP from S.Chanderbally") )
		about.set_logo_icon_name( "distributor-logo" )
  		about.connect( "response", lambda dialog, r: dialog.destroy() )
		about.show()


	def showPreferences(self, uicomponent, verb):
		os.system("gconf-editor /apps/mintMenu")

	def showMenuEditor(self, uicomponent, verb):
		os.system("alacarte")

	def showMenu(self, widget, event):
		
		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
			widget.emit_stop_by_name("button_press_event")
			origin = widget.window.get_origin()
			self.positionMenu(origin[0], origin[1], widget.size_request()[0], widget.allocation.y + widget.allocation.height)
			if self.toggle.get_active() == True:
				self.mainwin.hide()
				self.toggle.set_active(False)
			else:
				self.toggle.set_active(True)
				self.mainwin.show()				


		# show right click menu
		elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
			widget.emit_stop_by_name("button_press_event")
			self.create_menu()

		# allow middle click and drag
		elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 2:
			widget.emit_stop_by_name("button_press_event")
			self.toggle.set_active(False)
			self.mainwin.hide()

	def loseFocus(self, widget, event):
		if gtk.gdk.window_at_pointer() == None and self.mainwin.pinmenu == False:
			self.mainwin.hide()
			self.toggle.set_active(False)


	def positionMenu(self, x, y, w, h):
		# Get our own dimensions & position
		ourWidth  = self.mainwin.wTree.get_widget("window1").get_size()[0]
		ourHeight = self.mainwin.wTree.get_widget("window1").get_size()[1] + self.mainwin.offset

		# Get the dimensions/position of the widgetToAlignWith
		entryX, entryY = x, y
		entryWidth, entryHeight = w, h
		entryHeight = entryHeight + self.mainwin.offset

		# Get the screen dimensions
		screenHeight = gtk.gdk.screen_height()
		screenWidth = gtk.gdk.screen_width()

		if entryX + ourWidth < screenWidth:
			# Align to the left of the entry
			newX = entryX
		else:
			# Align to the right of the entry
			newX = (entryX + entryWidth) - ourWidth

		if entryY + entryHeight + ourHeight < screenHeight:
			# Align to the bottom of the entry
			newY = entryY + entryHeight
		else:
			newY = entryY - ourHeight

		# -"Move window"
		self.mainwin.wTree.get_widget("window1").move(newX, newY)

	# this callback is to create a context menu
	def create_menu(self):
		self.applet.setup_menu(self.propxml, self.verbs, None)

def menu_factory(applet, iid):
	MenuWin(applet, iid)
	applet.show_all()

	return True

if len(sys.argv) == 2 and sys.argv[1] == "run-in-window":
	main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	main_window.set_title("mintMenu")
	main_window.connect("destroy", gtk.main_quit)
	app = gnomeapplet.Applet()
	menu_factory(app, None)
	app.reparent(main_window)
	main_window.show_all()
	gtk.main()
	sys.exit()

gnomeapplet.bonobo_factory("OAFIID:GNOME_mintMenu_Factory",
                             gnomeapplet.Applet.__gtype__,
                             "mintMenu", "0", menu_factory)


