#!/usr/bin/env python

try:
	import pygtk
	pygtk.require( "2.0" )
except:
	pass
import gtk
import gtk.glade
import sys
import os
import gobject
import gconf
import gnomevfs
import string
import fnmatch
import gmenu
import re
import time
import gettext

from easyfiles import *
from easybuttons import *
from execute import *
from easygconf import *

gtk.glade.textdomain("mintMenu")
gtk.glade.bindtextdomain("mintMenu", "locale/")
_ = gettext.gettext

class Menu:
	def __init__( self, menu ):
		if isinstance( menu , gmenu.Directory):
			self.directory = menu
			self.tree = self.directory.get_tree()
		else:
			self.tree = gmenu.lookup_tree( menu )
			self.directory = self.tree.get_root_directory()

	def getMenus( self, parent=None ):
		if not parent:
			parent = self.directory
		if not parent:
			return

		for menu in parent.get_contents():
			if menu.get_type() == gmenu.TYPE_DIRECTORY and self.__isVisible( menu ):
				yield menu

	def getItems( self, recursive = False ):
		for item in self.directory.get_contents():
			if item.get_type() == gmenu.TYPE_ENTRY and item.get_desktop_file_id()[-19:] != '-usercustom.desktop' and self.__isVisible( item ):
				yield item
			elif recursive and item.get_type() == gmenu.TYPE_DIRECTORY and self.__isVisible( item ):
				for subitem in Menu( item ).getItems( True ):
					yield subitem

	def __isVisible( self, item ):
		if item.get_type() == gmenu.TYPE_ENTRY:
			return not ( item.get_is_excluded() or item.get_is_nodisplay() )
		if item.get_type() == gmenu.TYPE_DIRECTORY and len( item.get_contents() ):
			return True


class EditDialog:
	def __init__( self ):
		# Set the Glade file
		self.gladefile = os.path.join( os.path.dirname( __file__ ), "applications.glade" )

	def run( self, desktopFile ):
		"""This function will show the Edit Dialog"""

		desktopItem = gnomedesktop.item_new_from_uri( desktopFile, 0 )

		self.wTree = gtk.glade.XML( self.gladefile, "edit_dialog" )
		#Get the actual dialog widget
		self.dlg = self.wTree.get_widget( "edit_dialog" )

		self.wTree.get_widget( "edit_name_entry" ).set_text( desktopItem.get_localestring( gnomedesktop.KEY_NAME ) or "" )
		self.wTree.get_widget( "edit_genericname_entry" ).set_text( desktopItem.get_localestring( gnomedesktop.KEY_GENERIC_NAME ) or "" )
		self.wTree.get_widget( "edit_comment_entry" ).set_text( desktopItem.get_localestring( gnomedesktop.KEY_COMMENT ) or "" )
		self.wTree.get_widget( "edit_exec_entry" ).set_text( desktopItem.get_string( gnomedesktop.KEY_EXEC ) or "" )
		self.wTree.get_widget( "edit_icon_entry" ).set_text( desktopItem.get_string( gnomedesktop.KEY_ICON ) or "" )

		#run the dialog and store the response
		self.result = self.dlg.run()
		if self.result == -5:
			desktopItem.set_localestring( gnomedesktop.KEY_NAME, self.wTree.get_widget( "edit_name_entry" ).get_text() )
			desktopItem.set_localestring( gnomedesktop.KEY_GENERIC_NAME, self.wTree.get_widget( "edit_genericname_entry" ).get_text() )
			desktopItem.set_localestring( gnomedesktop.KEY_COMMENT, self.wTree.get_widget( "edit_comment_entry" ).get_text() )
			desktopItem.set_string( gnomedesktop.KEY_EXEC, self.wTree.get_widget( "edit_exec_entry" ).get_text() )
			desktopItem.set_string( gnomedesktop.KEY_ICON, self.wTree.get_widget( "edit_icon_entry" ).get_text() )

			desktopItem.save( desktopFile, 0 )


		self.dlg.destroy()
		return False

class pluginclass:
	TARGET_TYPE_TEXT = 80
	toButton = [ ( "text/uri-list", 0, TARGET_TYPE_TEXT ) ]

	def __init__( self, mintMenuWin, toggleButton ):

		self.mintMenuWin = mintMenuWin
		# Rebuild the buttonlist when applications are added or deleted
		for mainitems in ['applications.menu', 'settings.menu']:
			menu = Menu( mainitems )
			menu.tree.add_monitor( self.menuChanged, None )

		self.toggleButton = toggleButton	

		# The Glade file for the plugin
		self.gladefile = os.path.join( os.path.dirname( __file__ ), "applications.glade" )

		# Read GLADE file
		self.wTree = gtk.glade.XML( self.gladefile, "window1" )

		# These properties are NECESSARY to maintain consistency

		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget( "window1" )

		# Set 'heading' property for plugin
		self.heading = _("Applications")

		self.ApplicationTips = gtk.Tooltips()

		# This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget( "Applications" )

		# Items to get custom colors
		self.itemstocolor = [ self.wTree.get_widget( "viewport1" ), self.wTree.get_widget( "viewport2" ), self.wTree.get_widget( "viewport3" ), self.wTree.get_widget( "notebook2" ) ]

		# Unset all timers
		self.filterTimer = None
		self.menuChangedTimer = None

		# Hookup for text input
		self.content_holder.connect( "key-press-event", self.keyPress )


		self.wTree.get_widget( "applications_button_holder" ).connect( "drag_data_received", self.ReceiveCallback )
		self.wTree.get_widget( "applications_button_holder" ).drag_dest_set( gtk.DEST_DEFAULT_MOTION |
							 gtk.DEST_DEFAULT_HIGHLIGHT |
							 gtk.DEST_DEFAULT_DROP,
							 self.toButton, gtk.gdk.ACTION_COPY )
		self.wTree.get_widget( "button4" ).connect( "drag_data_received", self.ReceiveCallback )
		self.wTree.get_widget( "button4" ).drag_dest_set( gtk.DEST_DEFAULT_MOTION |
							 gtk.DEST_DEFAULT_HIGHLIGHT |
							 gtk.DEST_DEFAULT_DROP,
							 self.toButton, gtk.gdk.ACTION_COPY )

		self.wTree.get_widget( "button7" ).connect( "button_release_event", self.SearchWithButton )


		# Gconf stuff		

		self.gconf = EasyGConf( "/apps/mintMenu/plugins/applications/" )
		self.GetGconfEntries()

		self.gconf.notifyAdd( "execute_app", self.RegenPlugin )
		self.gconf.notifyAdd( "do_not_filter", self.RegenPlugin )
		self.gconf.notifyAdd( "icon_size", self.RegenPlugin )
		self.gconf.notifyAdd( "height", self.RegenPlugin )
		self.gconf.notifyAdd( "width", self.RegenPlugin )
		self.gconf.notifyAdd( "categories_mouse_over", self.RegenPlugin )
		self.gconf.notifyAdd( "swap_generic_name", self.RegenPlugin )
		self.gconf.notifyAdd( "categories_mouse_over", self.RegenPlugin )
		self.gconf.notifyAdd( "category_hower_delay", self.RegenPlugin )
		self.gconf.notifyAdd( "show_category_icons", self.RegenPlugin )
		self.gconf.notifyAdd( "show_application_comments", self.RegenPlugin )


		self.currentFavCol = 0

		self.content_holder.set_size_request( self.width, self.height )

#		start = time.time()
#		for i in range(0,5000):
#			desktopItem = gnomedesktop.item_new_from_uri( "/usr/share/applications/nautilus-c#omputer.desktop", 0 )
	#		desktopItem.get_localestring( gnomedesktop.KEY_NAME )
	#		desktopItem.get_localestring( gnomedesktop.KEY_GENERIC_NAME )
	#		desktopItem.get_localestring( gnomedesktop.KEY_COMMENT )
	#		desktopItem.get_localestring( gnomedesktop.KEY_EXEC )
	#		desktopItem.get_localestring( gnomedesktop.KEY_ICON )
	#		desktopItem.get_localestring( gnomedesktop.KEY_CATEGORIES )
 	#	print (time.time() - start)
	#	start = time.time()
	#	for i in range(0,5000):
	#		ParseDesktopFile( "/usr/share/applications/nautilus-computer.desktop" )
 	#	print (time.time() - start)

	def RegenPlugin( self, *args, **kargs ):
		# save old config - this is necessary because the app will notified when it sets the default values and you don't want the to reload itself several times
		oldexecapp = self.execapp
		oldcategories_mouse_over = self.categories_mouse_over
		oldwidth = self.width
		oldheight = self.height
		olddonotfilterapps = self.donotfilterapps
		oldiconsize = self.iconsize
		oldfaviconsize = self.faviconsize
		oldtotalrecent = self.totalrecent
		oldswapgeneric = self.swapgeneric
		oldshowcategoryicons = self.showcategoryicons
		oldcategoryhoverdelay = self.categoryhowerdelay
		oldsticky = self.sticky
		oldminimized = self.minimized
		oldsearchtool = self.searchtool
		oldicon = self.icon
		oldhideseparator = self.hideseparator
		oldshowapplicationcomments = self.showapplicationcomments

		self.GetGconfEntries()

		# if the config hasn't changed return
		if oldexecapp == self.execapp and oldcategories_mouse_over == self.categories_mouse_over and oldwidth == self.width and oldheight == self.height and olddonotfilterapps == self.donotfilterapps and oldiconsize == self.iconsize and oldfaviconsize == self.faviconsize and oldtotalrecent == self.totalrecent and oldswapgeneric == self.swapgeneric and oldshowcategoryicons == self.showcategoryicons and oldcategoryhoverdelay == self.categoryhowerdelay and oldsticky == self.sticky and oldminimized == self.minimized and oldsearchtool == self.searchtool and oldicon == self.icon and oldhideseparator == self.hideseparator and oldshowapplicationcomments == self.showapplicationcomments:
			return

		self.Todos()
		self.do_applications_file()
		self.RebuildPlugin()

	def GetGconfEntries( self ):

		self.execapp = 			self.gconf.get( "string", "execute_app", "nautilus" )
		self.categories_mouse_over = self.gconf.get( "bool", "categories_mouse_over", True )
		self.width = 			self.gconf.get( "int", "width", 380 )
		self.height = 			self.gconf.get( "int", "height", 410 )
		self.donotfilterapps = 	self.gconf.get( "bool", "do_not_filter", False )
		self.iconsize = 		self.gconf.get( "int", "icon_size", 2 )
		self.faviconsize = 		self.gconf.get( "int", "favicon_size", 3 )
		self.favCols	 = 		self.gconf.get( "int", "fav_cols", 1 )
		self.totalrecent = 		self.gconf.get( "int", "totalrecent", 0 )
		self.swapgeneric = 		self.gconf.get( "bool", "swap_generic_name", False )
		self.showcategoryicons = self.gconf.get( "bool", "show_category_icons", True)
		self.categoryhowerdelay = self.gconf.get( "int", "category_hower_delay", 150)
		self.showapplicationcomments = self.gconf.get( "bool", "show_application_comments", False)


		# Allow plugin to be minimized to the left plugin pane
		self.sticky 	= self.gconf.get( "bool", "sticky", False )
		self.minimized 	= self.gconf.get( "bool", "minimized", False )

		# Search tool
		self.searchtool = self.gconf.get( "string", "search_command", "beagle-search SEARCH_STRING" )

		# Plugin icon
		self.icon = self.gconf.get( "string", "icon", "applications-accessories" )

		# Hide vertical dotted separator
		self.hideseparator = self.gconf.get( "bool", "hide_separator", False )

	def SetHidden( self, state ):
		if state == True:
			self.gconf.set( "bool", "minimized", True )
		else:
			self.gconf.set( "bool", "minimized", False )

	def RebuildPlugin(self):
		self.content_holder.set_size_request( self.width, self.height )

	#Add from applications file
	def do_applications_file( self ):
		TestForDir( ".linuxmint/mintMenu/applications" )
		TestForFiles( ".linuxmint/mintMenu/applications.list" )
		TestForFiles( ".linuxmint/mintMenu/recentapps.list" )

		applicationsFile = open ( os.path.join( os.path.expanduser( "~" ), ".linuxmint/mintMenu/applications.list" ), "r" )
		applicationsList = applicationsFile.readlines()

		cols = self.wTree.get_widget( "applications_button_holder" ).get_children()

		for col in cols:
			for i in col.get_children():
				i.destroy()

		colIndex = 0

		for app in applicationsList :
			app = app.strip()
			if ( app[-8:] == '.desktop' ):
				try:

					FavButton = FavApplicationLauchner( app.strip(), self.faviconsize, self.swapgeneric )
					FavButton.show()
					FavButton.connect( "popup-menu", self.favPopup )
					FavButton.connect( "button_release_event", self.favPopup )
					FavButton.connect( "focus-in-event", self.scrollItemIntoView )
					FavButton.connect( "clicked", lambda w: self.mintMenuWin.hide()  )
					
					self.ApplicationTips.set_tip( FavButton, FavButton.getTooltip() )

					cols[colIndex].pack_start( FavButton, False )

				except:
					print "File in favorites not found: '" + app + "'"

			elif app == "separator":
				cols[colIndex].pack_start( self.seperator(), False )
			elif app == "space":
				cols[colIndex].pack_start( self.space(), False )

			colIndex = colIndex + 1
			if colIndex >= self.favCols:			
				colIndex = 0


	#Add a space
	def do_space( self, menuItem, widget, insertBefore ):
		Event1 = self.space()
		widget.parent.pack_start( Event1, False )

		count = 0
		for i in widget.parent.get_children():
			if widget == i:
				if insertBefore:
					widget.parent.reorder_child( Event1, count )
				else:
					widget.parent.reorder_child( Event1, count + 1 )
				break
			count = count + 1

		self.saveFavorites()

	def space( self ):
		Event1 = gtk.EventBox()
		Event1.set_size_request( -1, 20 )
		Event1.connect( "button_release_event", self.favPopup, False )
		Event1.desktopFile = "space"
		
		self.mintMenuWin.SetPaneColors( [ Event1 ] )
		Event1.show()

		return Event1

	def saveFavorites( self ):
		appListFile = open( os.path.expanduser( "~/.linuxmint/mintMenu/applications.list" ), "w" )
		cols = self.wTree.get_widget( "applications_button_holder" ).get_children()
		r = 0
		for col in cols:
			r = max( r, len(col.get_children()) )
		for i in range(r):
			for col in cols:
				if i < len(col.get_children()):
					appListFile.write( col.get_children()[i].desktopFile + "\n" )

		appListFile.close( )


	#Add a separator
	def do_separator( self, menuItem, widget, insertBefore ):

		Event1 = self.seperator()
		widget.parent.pack_start( Event1, False )

		count = 0
		for i in widget.parent.get_children():
			if widget == i:
				if insertBefore:
					widget.parent.reorder_child( Event1, count )
				else:
					widget.parent.reorder_child( Event1, count + 1 )
				break
			count = count + 1

		self.saveFavorites()

	def seperator( self ):
		sep1 = gtk.HSeparator()
		Event1 = gtk.EventBox()
		Event1.add( sep1 )
		Event1.set_size_request( -1, 20 )
		sep1.show()
		Event1.connect( "button_release_event", self.favPopup, False )
		Event1.desktopFile = "separator"

		self.mintMenuWin.SetPaneColors( [ Event1 ] )
		Event1.show()
		return Event1

	def ChangeTab( self, TabNum ):
		notebook = self.wTree.get_widget( "notebook2" )
		if TabNum == 0:
			notebook.set_current_page( 0 )
		elif TabNum == 1:
			notebook.set_current_page( 1 )

		self.focusSearchEntry()

	def Todos( self ):

		self.wTree.get_widget( "entry1" ).connect( "changed", self.Filter )
		self.wTree.get_widget( "entry1" ).connect( "activate", self.Search )
		self.wTree.get_widget( "button3" ).connect( "clicked", lambda widget: self.ChangeTab( 1 ) )
		self.wTree.get_widget( "button4" ).connect( "clicked", lambda widget: self.ChangeTab( 0 ) )

		favAppBtnHolder = self.wTree.get_widget( "applications_button_holder" )

		for child in favAppBtnHolder.get_children():
			child.destroy()
	
		for i in range(self.favCols):
			favAppBtnHolder.pack_start( gtk.VBox() )
		favAppBtnHolder.show_all()

		self.buildButtonList()

	def focusSearchEntry( self ):
		serachEntry = self.wTree.get_widget( "entry1" )

		# grab_focus() does select all text, as this is an unwanted behaviour we restore the old selection
		sel = serachEntry.get_selection_bounds()
		if len(sel) == 0: # no selection
			sel = ( serachEntry.get_position(), serachEntry.get_position() )
		serachEntry.grab_focus()
		serachEntry.select_region( sel[0], sel[1] )


	def menuChanged( self, x, y):
		# Wait a moment before updating the list, because it's
		# very likely that we recive a whole bunch of notifications.
		if self.menuChangedTimer:
			gobject.source_remove( self.menuChangedTimer )
			self.menuChangedTimer = None

		self.menuChangedTimer = gobject.timeout_add( 10, self.buildButtonList )

	def buildButtonList( self ):

		categoryBox = self.wTree.get_widget( "vbuttonbox2" )
		applicatonBox = self.wTree.get_widget( "vbuttonbox3" )

		categoryBox.set_size_request( self.width / 3, -1 )
		applicatonBox.set_size_request( self.width / 2, -1 )

		itemslist = []

		# Remove all old buttons
		for child in categoryBox.get_children():
			child.destroy()
		for child in applicatonBox.get_children():
			child.destroy()

		# Create "All Applications" button
		if self.showcategoryicons == True:
				categoryIcon = "distributor-logo"
		else:
				categoryIcon = ""

		CategoryButton = easyButton( categoryIcon, self.iconsize, [ _("All") ] )
		self.ApplicationTips.set_tip( CategoryButton, _("Show all applications") )
		if (self.categories_mouse_over == True):
			CategoryButton.connect( "enter", self.StartFilter, "" )
			CategoryButton.connect( "leave", self.StopFilter )
		else:
			CategoryButton.connect( "clicked", self.Filter, "" )

		CategoryButton.connect( "focus-in-event", self.categoryBtnFocus, "" )
		CategoryButton.show()

		categoryBox.pack_start( CategoryButton, False )

		for mainitems in ['applications.menu', 'settings.menu']:
			mymenu = Menu( mainitems )

			for child in mymenu.getMenus():

				if self.showcategoryicons == True:
			 		categoryIcon = child.get_icon()
				else:
					 categoryIcon = ""

				CategoryButton = easyButton( categoryIcon, self.iconsize, [ child.get_name() ] )
				self.ApplicationTips.set_tip( CategoryButton, child.get_name() )
		
				if (self.categories_mouse_over == True):
					CategoryButton.connect( "enter", self.StartFilter, child.get_name() )
					CategoryButton.connect( "leave", self.StopFilter )
				else:
					CategoryButton.connect( "clicked", self.Filter, child.get_name() )	
				CategoryButton.connect( "focus-in-event", self.categoryBtnFocus, child.get_name() )
				CategoryButton.show()

				categoryBox.pack_start( CategoryButton, False )

				for grandchild in Menu(child).getItems( True ):

					ItemButton = MenuApplicationLauchner( grandchild.get_desktop_file_path(), self.iconsize, child.get_name(), self.showapplicationcomments )
					self.ApplicationTips.set_tip( ItemButton, ItemButton.getTooltip() )
					ItemButton.connect( "button-release-event", self.menuPopup )
					ItemButton.connect( "focus-in-event", self.scrollItemIntoView )
					ItemButton.connect( "clicked", lambda w: self.mintMenuWin.hide()  )
					ItemButton.show()
					itemslist.append( ( string.upper( ItemButton.appName ), ItemButton ) )

		# Order alphabetically by name
		itemslist.sort()
		for i in itemslist:
			applicatonBox.pack_start( i[1], False )


	def categoryBtnFocus( self, widget, event, category ):
		self.scrollItemIntoView( widget )
		self.StartFilter( widget, category )

	def StartFilter( self, widget, category ):
		# if there is a timer for a different category running stop it
		if self.filterTimer:
			gobject.source_remove( self.filterTimer )
		self.filterTimer = gobject.timeout_add( self.categoryhowerdelay, self.Filter, widget, category )

	def StopFilter( self, widget ):
		if self.filterTimer:
			gobject.source_remove( self.filterTimer )
			self.filterTimer = None

	def Filter( self, widget, category = None ):
		self.filterTimer = None

		start = time.time()

		if widget == self.wTree.get_widget( "entry1" ):
			if self.donotfilterapps:
				widget.set_text( '' )
			else:
				self.ChangeTab( 1 )
		   		text = widget.get_text()
		   		for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
		   			i.filterText( text )
		else:
			if category == "":
	   			for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
						i.show()
			else:
	   			for i in self.wTree.get_widget( "vbuttonbox3" ).get_children():
					i.filterCategory( category )


	   		for i in self.wTree.get_widget( "vbuttonbox2" ).get_children():
	   			i.set_relief( gtk.RELIEF_NONE )
	   		widget.set_relief( gtk.RELIEF_HALF )
			widget.grab_focus()

	   		self.wTree.get_widget( "entry1" ).set_text( '' )

		print "Filtertime: ", (time.time() - start), "s"	

	# Forward all text to the search box
	def keyPress( self, widget, event ):

		if event.string.strip() != "" or event.keyval == gtk.keysyms.BackSpace:
	  		self.wTree.get_widget( "entry1" ).event( event )
			return True

		return False
			

	def favPopup( self, widget, ev, enableEdit = True ):
		if ev.button == 3:
			if ev.y > widget.get_allocation().height / 2:
				insertBefore = False
			else:
				insertBefore = True
			self.mTree = gtk.glade.XML( self.gladefile, "menu1" )

			self.mTree.get_widget( "edit1" ).connect( "activate", self.EditButton, widget, ev, self.mTree )
			if not enableEdit:
				self.mTree.get_widget( "edit1" ).set_sensitive( False )
			self.mTree.get_widget( "remove1" ).connect( "activate", self.DelButton, widget, widget, ev, self.mTree )
			self.mTree.get_widget( "move_up1" ).connect( "activate", self.MoveUp, widget, self.mTree )
			self.mTree.get_widget( "move_down1" ).connect( "activate", self.MoveDown, widget, self.mTree )
			self.mTree.get_widget( "insert_space1" ).connect( "activate", self.do_space, widget, insertBefore )
			self.mTree.get_widget( "insert_separator1" ).connect( "activate", self.do_separator, widget, insertBefore )
			self.mTree.get_widget( "menu1" ).popup( None, None, None, ev.button, ev.time )

	def menuPopup( self, widget, event ):
		if event.button == 3:
			self.mTree = gtk.glade.XML( self.gladefile, "menu2" )
			self.mTree.get_widget( "menuitem1" ).connect( "activate", self.AddFavourite, widget, self.mTree )
			self.mTree.get_widget( "menu2" ).popup( None, None, None, event.button, event.time )

	def DelButton( self, a, widget, itemclicked, event, menutogo, *args ):
		if itemclicked.desktopFile not in ["space", "separator"]:
			os.remove( itemclicked.desktopFile )
		itemclicked.destroy()
		menutogo.get_widget( "menu1" ).destroy()

		self.saveFavorites()

		self.do_applications_file()


	def EditButton( self, a, widget, event, menutogo ):
		EditDlg = EditDialog()
		result = EditDlg.run( widget.desktopFile )

		self.do_applications_file()
		menutogo.get_widget( "menu1" ).destroy()


	def MoveUp( self, a, widget, menutogo ):
		count = 0
		for i in widget.parent.get_children():
			if widget == i and count != 0:
				widget.parent.reorder_child( widget, count - 1 )
				break
			count = count + 1
		menutogo.get_widget( "menu1" ).destroy()
		self.saveFavorites()

		self.do_applications_file()

	def MoveDown( self, a, widget, menutogo ):
		count = 0
		for i in widget.parent.get_children():
			if widget == i:
				widget.parent.reorder_child( widget, count + 1 )
				break
			count = count + 1
		menutogo.get_widget( "menu1" ).destroy()
		self.saveFavorites()

		self.do_applications_file()

	def AddToFavourites( self, location ):
		# Don't add favorites to the favorites
		if fnmatch.fnmatch( GetFilePath( location ), os.path.join( os.path.expanduser( "~" ), ".linuxmint/mintMenu/applications/*" ) ):
			return

		ButtonIcon = None
		# For Folders and Network Shares
		ButtonLocation = string.join( location.split( "%20" ) )

		if ButtonLocation[:4] == 'file':
			ButtonIcon = "gnome-fs-directory"

		if ButtonLocation[:3] == 'smb' or ButtonLocation[:3] == 'ssh' or ButtonLocation[:7] == 'network':
			ButtonIcon = "gnome-fs-network"

		if ButtonIcon != None:
			locationName = [ButtonLocation.split( '/' )[-1:][0]]
			if locationName == [""]:
				locationName = "/"


		#For Special locations
		if location == 'x-nautilus-desktop:///computer':
			location = "/usr/share/applications/nautilus-computer.desktop"

		elif location == 'x-nautilus-desktop:///home':
			location =  "/usr/share/applications/nautilus-home.desktop"

		elif location == 'x-nautilus-desktop:///network':
			location = "/usr/share/applications/network-scheme.desktop"

		if location:

			FileExists = False
			FileData = GetFilePath( location )
			clean1 = FileData.replace( "/", "-" )
			DroppedFile = clean1.replace( " ", "" )
			DroppedFile = ".linuxmint/mintMenu/applications/" + DroppedFile
			DroppedFile = os.path.join( os.path.expanduser( "~" ), DroppedFile )

			if not os.path.exists( DroppedFile ):
				print "Application added - " + DroppedFile
				item = gnomedesktop.item_new_from_uri( location, 0 )
				item.save( DroppedFile, False )
				WriteListFile( ".linuxmint/mintMenu/applications.list", DroppedFile, "a" )
				self.do_applications_file()

	def AddFavourite( self, widget, button, menu ):
		self.AddToFavourites( button.desktopFile )
		self.do_applications_file()

	def ReceiveCallback( self, widget, context, x, y, selection, targetType, time ):
		if targetType == self.TARGET_TYPE_TEXT:
			for uri in selection.get_uris():
				self.AddToFavourites( uri )

	def Search( self, widget ):
		text = self.wTree.get_widget( "entry1" ).get_text().strip()
		if text != "":
			self.wTree.get_widget( "entry1" ).set_text( '' )
			self.mintMenuWin.wTree.get_widget( "window1" ).hide()
			self.toggleButton.set_active(False)
			fullstring = self.searchtool.replace( "SEARCH_STRING", text )
			newstring = fullstring.split()
			Execute( widget, newstring )

	def SearchWithButton( self, widget, event ):
		self.Search( widget )

	def do_plugin( self ):
		self.Todos()
		self.do_applications_file()
		
		# If there are no favorites show the apps tab
		cols = self.wTree.get_widget( "applications_button_holder" ).get_children()
		r = 0
		for col in cols:
			r = max( r, len(col.get_children()) )

		if not r:
			self.ChangeTab( 1 )

	# Scroll button into view
	def scrollItemIntoView( self, widget, event = None ):
		viewport = widget.parent
		while not isinstance( viewport, gtk.Viewport ):
			if not viewport.parent:
				return
			viewport = viewport.parent
		aloc = widget.get_allocation()
		viewport.get_vadjustment().clamp_page(aloc.y, aloc.y + aloc.height)



#---------------------------------------------------------------------------------------#
# mintMenuconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins USPconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "applications.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in USPconfig
		self.heading = "Applications"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/mintMenu/plugins/applications'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/mintMenu/plugins/applications', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_GenFontSizeSpin_changed" : self.genfontspin,
				"on_NameFontSizeSpin_changed" : self.namefontspin,
				"on_AppIconSizeSpin_changed" : self.appiconspin,
				"on_AppsHSpin_changed" : self.apph,
				"on_ShowAllAppsChkBtn_toggled" : self.showallapps,
				"on_MoreAppsChkBtn_toggled" : self.moreapps,
				"on_GenChkBtn_toggled" : self.genchk,
				"on_SrchCmdEntry_changed" : self.srchcmd,
				"on_HideSrchChkBtn_toggled" : self.hidesrch,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_AppIconEntry_changed" : self.appiconchange,
				"on_StickyChkBtn_toggled" : self.appsticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("GenFontSizeSpin").set_value(self.gconf.get(self.client,'int','/apps/mintMenu/plugins/applications/appgeneric_font_size',9))
		self.wTree.get_widget("NameFontSizeSpin").set_value(self.gconf.get(self.client,'int','/apps/mintMenu/plugins/applications/appname_font_size',11))
		self.wTree.get_widget("AppIconSizeSpin").set_value(self.gconf.get(self.client,'int','/apps/mintMenu/plugins/applications/apps_icon_size',11))
		self.wTree.get_widget("AppsHSpin").set_value(self.gconf.get(self.client,'int','/apps/mintMenu/plugins/applications/height',410))
		self.wTree.get_widget("ShowAllAppsChkBtn").set_active(self.gconf.get(self.client,'bool','/apps/mintMenu/plugins/applications/show_allapps_on_start',False))
		self.wTree.get_widget("MoreAppsChkBtn").set_active(self.gconf.get(self.client,'bool','/apps/mintMenu/plugins/applications/show_more_apps_button',True))
		self.wTree.get_widget("GenChkBtn").set_active(self.gconf.get(self.client,'bool','/apps/mintMenu/plugins/applications/swap_generic_name',False))
		self.wTree.get_widget("SrchCmdEntry").set_text(self.gconf.get(self.client,'string','/apps/mintMenu/plugins/applications/search_command','beagle-search SEARCH_STRING'))
		self.wTree.get_widget("HideSrchChkBtn").set_active(self.gconf.get(self.client,'bool','/apps/mintMenu/plugins/applications/hide_search_box',False))
		self.wTree.get_widget("HideSepChkBtn").set_active(self.gconf.get(self.client, 'bool', '/apps/mintMenu/plugins/applications/hide_separator',False))
		self.wTree.get_widget("AppIconEntry").set_text(self.gconf.get(self.client,'string','/apps/mintMenu/plugins/applications/icon','gnome-applications'))
		self.wTree.get_widget("StickyChkBtn").set_active(self.gconf.get(self.client,'bool','/apps/mintMenu/plugins/applications/sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def genfontspin(self, widget, **kargs):
		self.client.set_int('/apps/mintMenu/plugins/applications/appgeneric_font_size',int(self.wTree.get_widget("GenFontSizeSpin").get_value()))

	def namefontspin(self, widget, **kargs):
		self.client.set_int('/apps/mintMenu/plugins/applications/appname_font_size',int(self.wTree.get_widget("NameFontSizeSpin").get_value()))

	def apph(self, widget, **kargs):
		self.client.set_int('/apps/mintMenu/plugins/applications/height',int(self.wTree.get_widget("AppsHSpin").get_value()))

	def appiconspin(self, widget, **kargs):
		self.client.set_int('/apps/mintMenu/plugins/applications/apps_icon_size',int(self.wTree.get_widget("AppIconSizeSpin").get_value()))

	def showallapps(self, widget, **kargs):
		if self.wTree.get_widget("ShowAllAppsChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/show_allapps_on_start',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/show_allapps_on_start',False)

	def moreapps(self, widget, **kargs):
		if self.wTree.get_widget("MoreAppsChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/show_more_apps_button',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/show_more_apps_button',False)

	def genchk(self, widget, **kargs):
		if self.wTree.get_widget("GenChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/swap_generic_name',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/swap_generic_name',False)

	def hidesrch(self, widget, **kargs):
		if self.wTree.get_widget("HideSrchChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/hide_search_box',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/hide_search_box',False)

	def srchcmd(self, widget, **kargs):
		self.client.set_string('/apps/mintMenu/plugins/applications/search_command',self.wTree.get_widget("SrchCmdEntry").get_text())

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/hide_separator',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/hide_separator',False)

	def appiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("AppIconEntry").get_text()
		self.client.set_string('/apps/mintMenu/plugins/applications/icon',self.wTree.get_widget('AppIconEntry').get_text())

	def appsticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('/apps/mintMenu/plugins/applications/sticky',True)
		else:
			self.client.set_bool('/apps/mintMenu/plugins/applications/sticky',False)
