#!/usr/bin/env python

import gtk
import gtk.glade
import pango
import gnomedesktop
import gobject
from easyfiles import *
import re
from execute import *
import os.path

class IconManager(gobject.GObject):

	__gsignals__ = {
 		"changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () )
	}

	def __init__( self ):
		gobject.GObject.__init__( self )
		self.icons = { }

		# Some apps don't put a default icon in the default theme folder, so we will search all themes
		def createTheme( d ):
			theme = gtk.IconTheme()
			theme.set_custom_theme( d )
			return theme
		
		self.themes = map(  createTheme, [ d for d in os.listdir( "/usr/share/icons" ) if os.path.isdir( os.path.join( "/usr/share/icons", d ) ) ] )
		
		defaultTheme = gtk.icon_theme_get_default()
		
		# Themes witrh the same content as the default them aren't needed
		self.themes = [ theme for theme in self.themes if  theme.list_icons() != defaultTheme.list_icons() ]
				
		self.themes.insert( 0, defaultTheme )

		# Listen for changes in the themes
		for theme in self.themes:
			theme.connect("changed", self.themeChanged )


	def getIcon( self, iconName, iconSize ):
		if not iconName:
			return None

		if isinstance(iconSize, int):
			if iconSize >= 4:
				iconSize = gtk.ICON_SIZE_DIALOG
			elif iconSize == 3:
				iconSize = gtk.ICON_SIZE_DND
			elif iconSize == 2:
				iconSize = gtk.ICON_SIZE_BUTTON
			elif iconSize == 1:
				iconSize = gtk.ICON_SIZE_MENU
			elif iconSize <= 0:
				return None

		[ iconWidth , iconHeight ] = gtk.icon_size_lookup(iconSize)

		if iconName[-4:] in [".png", ".xpm", ".svg", ".gif"] and iconName[0:1] == "/":
			if FileExists( iconName ):
				icon = gtk.gdk.pixbuf_new_from_file_at_size( iconName, iconWidth, iconHeight )
			else:
				return None
		else:
			if iconName[-4:] in [".png", ".xpm", ".svg", ".gif"]:
				iconName = iconName[:-4]
			tmp = None
			for theme in self.themes:
				tmp = theme.lookup_icon( iconName, iconWidth, 0)
				if tmp:
					break
			if tmp:
				icon = tmp.load_icon()
			else:
				return None

		# if the actual icon size is to far from the desired size resize it
		if ( icon.get_width() - iconWidth ) > 5 or ( icon.get_height() - iconHeight ) > 5:
			if icon.get_width() > icon.get_height():
				icon = icon.scale_simple( iconWidth, icon.get_height() * iconWidth / icon.get_width(), gtk.gdk.INTERP_BILINEAR )
			else:
				icon = icon.scale_simple( icon.get_width() * iconHeight / icon.get_height(), iconWidth, gtk.gdk.INTERP_BILINEAR )

		return icon

	def themeChanged( self, theme ):
		self.emit( "changed" )


class easyButton( gtk.Button ):

	def __init__( self, iconName, iconSize, labels = None, buttonWidth = -1, buttonHeight = -1 ):

		gtk.Button.__init__( self )

		self.iconName = iconName
		self.iconSize = iconSize
		self.showIcon = True

		self.set_relief( gtk.RELIEF_NONE )
		self.set_size_request( buttonWidth, buttonHeight )

		Align1 = gtk.Alignment( 0, 0.5, 1.0, 0 )
		HBox1 = gtk.HBox()
		self.labelBox = gtk.VBox( False, 2 )

		icon = self.getIcon( self.iconSize )
		if icon:
			self.buttonImage = gtk.Image()
			self.buttonImage.set_from_pixbuf( icon )
			self.buttonImage.show()
			HBox1.pack_start( self.buttonImage, False, False, 5 )

		if labels:
			for label in labels:
				if isinstance( label, str ):
					self.addLabel( label )
				elif isinstance( label, list ):
					self.addLabel( label[0], label[1] )

		self.labelBox.show()
		HBox1.pack_end( self.labelBox )
		HBox1.show()
		Align1.add( HBox1 )
		Align1.show()
		self.add( Align1 )

		# Reload icons when the theme changed
		iconManager.connect("changed", self.themeChanged )


	def addLabel( self, text, styles = None ):
		label = gtk.Label( text )

		if styles:
			labelStyle = pango.AttrList()
			for attr in styles:
				labelStyle.insert( attr )
			label.set_attributes( labelStyle )

		label.set_ellipsize( pango.ELLIPSIZE_END )
		label.set_alignment( 0.0, 1.0 )
		label.show()

		self.labelBox.pack_start( label )


	def getIcon ( self, iconSize ):
		icon = iconManager.getIcon( self.iconName, iconSize )
		if not icon:
			icon = iconManager.getIcon( "application-default-icon", iconSize )

		return icon

	# IconTheme changed, setup new button icons
	def themeChanged( self, theme ):
		icon = self.getIcon( self.iconSize )
		if icon:
			self.buttonImage.set_from_pixbuf( icon )

class ApplicationLauncher( easyButton ):

	def __init__( self, desktopFile, iconSize ):

		
		self.desktopFile    = desktopFile
		self.desktopItem    = gnomedesktop.item_new_from_uri( desktopFile, 0 )
		self.appName        = self.desktopItem.get_localestring( gnomedesktop.KEY_NAME ) or ""
		self.appGenericName = self.desktopItem.get_localestring( gnomedesktop.KEY_GENERIC_NAME ) or ""
		self.appComment     = self.desktopItem.get_localestring( gnomedesktop.KEY_COMMENT ) or ""
		self.appExec        = self.desktopItem.get_localestring( gnomedesktop.KEY_EXEC ) or ""
		appIconName         = self.desktopItem.get_localestring( gnomedesktop.KEY_ICON ) or ""
		self.appCategorys   = self.desktopItem.get_localestring( gnomedesktop.KEY_CATEGORIES ) or ""
		easyButton.__init__( self, appIconName, iconSize )
		self.setupLabels()

		# Drag and Drop
		self.connect( "drag_data_get", self.dragDataGet )
		self.drag_source_set( gtk.gdk.BUTTON1_MASK  , [ ( "text/plain", 0, 100 ), ( "text/uri-list", 0, 101 ) ], gtk.gdk.ACTION_COPY )

		icon = self.getIcon( 3 )
		if icon:
			self.drag_source_set_icon_pixbuf( icon )

		self.connect("focus-in-event", lambda widget, e: widget.set_relief( gtk.RELIEF_HALF ) )
		self.connect("focus-out-event", lambda widget, e: widget.set_relief( gtk.RELIEF_NONE ) )
#		self.connect("enter-notify-event", lambda widget, e: widget.grab_focus() )
		self.connect("clicked", lambda widget: self.execute() )

	def setupLabels( self ):
		self.addLabel( self.appName )

	def filterText( self, text ):
		keywords = text.lower().split()
		appName = self.appName.lower()
		appGenericName = self.appGenericName.lower()
		appComment = self.appComment.lower()
		appExec = self.appExec.lower()
		for keyword in keywords:
			if keyword != "" and appName.find( keyword ) == -1 and appGenericName.find( keyword ) == -1 and appComment.find( keyword ) == -1 and appExec.find( keyword ) == -1:
				self.hide()
				return
		
		self.show()

	def getTooltip( self ):
		tooltip = self.appName
		if self.appComment != "":
			tooltip = tooltip + "\n" + self.appComment

		return tooltip

	

	def dragDataGet( self, widget, context, selection, targetType, eventTime ):
		if targetType == 100: # text/plain
			selection.set_text( "'" + self.desktopFile + "'", -1 )
		elif targetType == 101: # text/uri-list
			selection.set_uris( [ "file://" + self.desktopFile ] )

	def execute( self ):	
		self.desktopItem.launch( [ ] )

	# IconTheme changed, setup new icons for button and drag 'n drop
	def themeChanged( self, theme ):
		easyButton.themeChanged( self, theme )

		icon = self.getIcon( 3 )
		if icon:
			self.drag_source_set_icon_pixbuf( icon )


class MenuApplicationLauchner( ApplicationLauncher ):

	def __init__( self, desktopFile, iconSize, category, showComment ):

		self.showComment = showComment
		self.appCategory = category

		ApplicationLauncher.__init__( self, desktopFile, iconSize )


	def filterCategory( self, category ):
		if self.appCategory == category:
			self.show()
		else:
			self.hide()

	def setupLabels( self ):
		if self.showComment and self.appComment != "":
			if self.iconSize <= 2:
				self.addLabel( self.appName, [ pango.AttrScale( pango.SCALE_SMALL, 0, -1 ) ] )
				self.addLabel( self.appComment, [ pango.AttrScale( pango.SCALE_X_SMALL, 0, -1 ) ] )
			else:
				self.addLabel( self.appName )
				self.addLabel( self.appComment, [ pango.AttrScale( pango.SCALE_SMALL, 0, -1 ) ] )
		else:
			self.addLabel( self.appName )

class FavApplicationLauchner( ApplicationLauncher ):

	def __init__( self, desktopFile, iconSize, swapGeneric = False ):

		self.swapGeneric = swapGeneric

		ApplicationLauncher.__init__( self, desktopFile, iconSize )

	def setupLabels( self ):
		if self.appGenericName:
			if self.swapGeneric:
				self.addLabel( self.appName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
				self.addLabel( self.appGenericName )
			else:
				self.addLabel( self.appGenericName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
				self.addLabel( self.appName )
		else:
			if self.appComment != "":
				self.addLabel( self.appName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
				self.addLabel( self.appComment )
			else:
				self.addLabel( self.appName )

iconManager = IconManager()
