#!/usr/bin/env python

import gconf


class EasyGConf:

	def __init__( self, gconfDir = None, client = None ):

		if not client:
			client = gconf.client_get_default()
		
		self.client = client
		self.gconfDir = gconfDir

		self.client.add_dir( self.gconfDir[:-1], gconf.CLIENT_PRELOAD_NONE )

	def get( self, type, key, default = None ):

		if key[0] != "/":
			key = self.gconfDir + key


		#check if key exists, if not set and return default value
		tempkey = self.client.get( key )
		if tempkey == None:
			self.set( type, key, default )
			return default
			

		if type == "bool":
			return self.client.get_bool( key )
		if type == "string":
			return self.client.get_string( key )
		if type == "int":
			return self.client.get_int( key )
		if type == "color":
			color = self.client.get_string( key )
			if not self.evalColor( color ):
				self.set( type, key, default )
				return default

			return color

		t = type.split("-")
		if len(t) == 2 and t[0] == "list":
			return self.client.get_list( key, t[1] )

		return self.client.get( key )


	def set( self, type, key, value ):

		if key[0] != "/":
			key = self.gconfDir + key

		if type == "bool":
			return self.client.set_bool( key, value )

		if type == "string":
			return self.client.set_string( key, value )

		if type == "int":
			return self.client.set_int( key, value )

		if type == "color":
			if self.evalColor( value ):
				return self.client.set_string( key, value )
			else:
				return self.client.set_string( "#ffffff", value )

		t = type.split("-")
		if len(t) == 2 and t[0] == "list":
			return self.client.set_list( key, t[1], value )

		return self.client.set( key, value )


	def notifyAdd( self, key, callback, gconfDir = None ):
		if key[0] != "/":
			key = self.gconfDir + key

		return self.client.notify_add( key, callback )

	def evalColor(self, colorToTest ):
		if colorToTest[0] != '#' or len( colorToTest ) != 7:
			return False
		for i in colorToTest[1:]:
			if i not in ['a', 'A', 'b', 'B', 'c', 'C', 'd', 'D', 'e', 'E', 'f', 'F', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9']:
				return False
		return True

