# User USP plugin

import gtk
import gtk.glade
import sys
import os
import gobject
import datetime
import gconf
import pango
import commands
from execute import Execute
from easygconf import SetGconf

class pluginclass:
    """This is the main class for the plugin"""
    """It MUST be named pluginclass"""

    def __init__(self):

	#The Glade file for the plugin
        self.gladefile = os.path.join(os.path.dirname(__file__), "mintMenuUser.glade")

        #Read GLADE file
        self.wTree = gtk.glade.XML(self.gladefile,"window1")

        #These properties are NECESSARY to maintain consistency
	    #throughout USP

        #Set 'window' property for the plugin (Must be the root widget)
        self.window = self.wTree.get_widget("window1")

        #Set 'heading' property for plugin
        self.heading = "User"

        #This should be the first item added to the window in glade
        self.content_holder = self.wTree.get_widget("eventbox1")

        #Specify plugin width
        self.width = 150

            #Plugin icon
        self.icon = 'system-users'

        self.gconf_dir = '/apps/usp/plugins/user'
        self.client = gconf.client_get_default()
        self.client.add_dir('/apps/usp/plugins/user', gconf.CLIENT_PRELOAD_NONE)
        self.client.notify_add('/apps/usp/plugins/user/settings_font_size',self.RegenPlugin)
        self.client.notify_add('/apps/usp/plugins/user/settings_font_bold',self.RegenPlugin)
        self.client.notify_add('/apps/usp/plugins/datetime/settings_date_sep',self.RegenPlugin)
        self.client.notify_add('/apps/usp/plugins/datetime/eu_date_format',self.RegenPlugin)
        self.client.notify_add( '/apps/usp/plugins/user/height', self.RegenPlugin )
        self.client.notify_add( '/apps/usp/plugins/user/width', self.RegenPlugin )


        gobject.timeout_add(1000, self.showuser)

    def RegenPlugin(self, *args, **kargs):
        self.GetGconfEntries()
        self.SetSystemStyle(["UserLabel","UserBelowLabel"])
        self.RebuildPlugin()

    def RebuildPlugin(self):
        self.content_holder.set_size_request(self.width, self.height )

    def SetupEvents(self):
        #Connect event handlers
        dic = {"on_window1_destroy" : gtk.main_quit,
               "on_ChooserBtn_clicked" : self.FaceChooser}
        self.wTree.signal_autoconnect(dic)

    def GetGconfEntries(self):
        self.client = gconf.client_get_default()
        self.fontsize = SetGconf(self.client,'int','/apps/usp/plugins/user/settings_font_size',14)
        self.fontbold = SetGconf(self.client,'bool','/apps/usp/plugins/user/settings_font_bold', False)
        self.namebelow = SetGconf(self.client,'bool','/apps/usp/plugins/user/name_below', False)
        self.hidelogged = SetGconf(self.client,'bool','/apps/usp/plugins/user/hide_logged_on', False)
        self.datesep = SetGconf(self.client,'string','/apps/usp/plugins/datetime/settings_date_sep','-')
        self.eudate = SetGconf(self.client,'bool','/apps/usp/plugins/datetime/eu_date_format', False)
        self.width = SetGconf( self.client, "int", "/apps/usp/plugins/user/width", 180 )
        self.height = SetGconf( self.client, "int", "/apps/usp/plugins/user/height", 200 )
        self.sticky = SetGconf( self.client, "bool", "/apps/usp/plugins/user/sticky", False )

    def FaceChooser(self, widget):
        os.system('gdmphotosetup')
	self.showuser()
        

    def SetSystemStyle(self,items):
        # Set Font styles
        HeadingStyle = pango.AttrList()
        attr = pango.AttrSize(self.fontsize*1000,0,-1)
        HeadingStyle.insert(attr)
        if self.fontbold == 0:
            attr = pango.AttrWeight(pango.WEIGHT_NORMAL,0,-1)
        else:
            attr = pango.AttrWeight(pango.WEIGHT_BOLD,0,-1)
            HeadingStyle.insert(attr)

        for i in range(len(items)):
	        self.wTree.get_widget(items[i]).set_attributes(HeadingStyle)

    def timetosecs(s):
        hms = s.split(":") # [hh, mm, ss]
        secs = 0
        for t in hms:
        	secs = secs * 60 + int(t)
        return secs

    def secstotime(secs):
        hms = []
        while secs:
        	hms.append(str(secs % 60))
        	secs = secs // 60
        return ":".join(hms)

    #Actually do something with the plugin
    def showuser(self):
        facepic = os.path.join(os.path.expanduser("~"), '.face')
        self.wTree.get_widget('Face').set_from_file(facepic)
        #username = os.popen('whoami').read()
        userlist = os.popen('who -s').read().split()
        datelogged = userlist[2].split("-")
        if self.eudate == 0:
        	showdate=datelogged[1]+self.datesep+datelogged[2]+self.datesep+datelogged[0]
        else:
        	showdate=datelogged[2]+self.datesep+datelogged[1]+self.datesep+datelogged[0]

        # Show User
        self.wTree.get_widget('UserLabel').set_label(userlist[0])
        self.wTree.get_widget('UserBelowLabel').set_label(userlist[0])
        # Show User Date Logged On
        self.wTree.get_widget('DateLabel').set_label("Logon Date : "+showdate)
        self.wTree.get_widget('DateBelowLabel').set_label("Logon Date : "+showdate)
        # Show User Time Logged On
        self.wTree.get_widget('TimeLabel').set_label("Logon Time : "+userlist[3])
        self.wTree.get_widget('TimeBelowLabel').set_label("Logon Time : "+userlist[3])
        # Show/Hide Date and Time Labels
        if self.hidelogged == True:
        	self.wTree.get_widget("DateLabel").hide()
        	self.wTree.get_widget("DateBelowLabel").hide()
        	self.wTree.get_widget("TimeLabel").hide()
        	self.wTree.get_widget("TimeBelowLabel").hide()
        else:
        	self.wTree.get_widget("DateLabel").show()
        	self.wTree.get_widget("DateBelowLabel").show()
        	self.wTree.get_widget("TimeLabel").show()
        	self.wTree.get_widget("TimeBelowLabel").show()
        # Sort Out Photo Position By Hiding VBox
        if self.namebelow == True:
        	self.wTree.get_widget("vbox1").hide()
        	self.wTree.get_widget("vbox2").show()
        else:
        	self.wTree.get_widget("vbox1").show()
        	self.wTree.get_widget("vbox2").hide()


    def do_plugin( self ):
        self.RegenPlugin()
        self.SetupEvents()
