#!/usr/bin/env python

import gtk
import gtk.glade
import sys
import os
import gobject
import gconf
import gnomevfs
import string
import gettext

from easyfiles import *
from easybuttons import *
from execute import *
from easygconf import *

gtk.glade.textdomain("mintMenu")
gtk.glade.bindtextdomain("mintMenu", "locale/")
_ = gettext.gettext

class pluginclass:
	def __init__( self, mintMenuWin, toggleButton ):

		self.mintMenuWin = mintMenuWin
		self.toggleButton = toggleButton
		
		# The Glade file for the plugin
		self.gladefile = os.path.join( os.path.dirname( __file__ ), "places.glade" )

		# Read GLADE file
		self.wTree = gtk.glade.XML( self.gladefile, "window1" )
		self.PlaceTips = gtk.Tooltips()
		# These properties are NECESSARY to maintain consistency

		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget( "window1" )
		#dic = {"on_window1_destroy" : gtk.main_quit}
		#self.wTree.signal_autoconnect(dic)

		# Set 'heading' property for plugin
		self.heading = _("Places")

		# This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget( "Places" )

		# Items to get custom colors
		self.itemstocolor = [self.wTree.get_widget( "viewport2" )]

		# Gconf stuff
		self.gconf = EasyGConf( "/apps/mintMenu/plugins/places/" )

		self.gconf.notifyAdd( "icon_size", self.RegenPlugin )
		self.gconf.notifyAdd( "height", self.RegenPlugin )
		self.gconf.notifyAdd( "width", self.RegenPlugin )


		self.GetGconfEntries()

	def RegenPlugin( self, *args, **kargs ):
		self.GetGconfEntries()
		#self.RebuildPlugin()
		self.ClearAll()
		self.do_standard_places()
		#self.do_mounted_volumes()
		#self.do_spacer()
		#self.do_gtk_bookmarks()

	def GetGconfEntries( self ):

		self.width = self.gconf.get( "int", "width", 200 )
		self.height = self.gconf.get( "int", "height", 180 )
		self.execapp = self.gconf.get( "string", "execute_app", "nautilus" )
		self.desktop_default = string.join( ["nautilus", os.path.join( os.path.expanduser( "~" ), "Desktop" )] )
		self.desktop = self.gconf.get( "string", "desktop_cmd", self.desktop_default ).split()
		self.home_default = string.join( ["nautilus", os.path.expanduser( "~" )] )
		self.home = self.gconf.get( "string", "home_cmd", self.home_default ).split()
		self.network = self.gconf.get( "string", "network_cmd", "nautilus --no-desktop network:" ).split()
		self.computer = self.gconf.get( "string", "computer_cmd", "nautilus --no-desktop computer:" ).split()
		self.trash = self.gconf.get( "string", "trash_cmd", "nautilus --no-desktop trash:" ).split()
		self.iconsize = self.gconf.get( "int","icon_size",2)

		# Hide vertical dotted separator
		self.hideseparator = self.gconf.get( "bool", "hide_separator", False )
		# Plugin icon
		self.icon = self.gconf.get( "string", 'icon', "gnome-fs-directory.png" )
		# Allow plugin to be minimized to the left plugin pane
		self.sticky = self.gconf.get( "bool", "sticky", False )
		self.minimized = self.gconf.get( "bool", "minimized", False )

	def SetHidden( self, state ):
		if state == True:
			self.gconf.set( "bool", "minimized", True )
		else:
			self.gconf.set( "bool", "minimized", False )

	def RebuildPlugin(self):
		self.content_holder.set_size_request(self.width, self.height )

	def ClearAll(self):
		for i in self.wTree.get_widget( "places_button_holder" ).get_children():
			i.destroy()
		for i in self.wTree.get_widget( "editable_button_holder" ).get_children():
			i.destroy()

	#Add bookmarks from Bookmarks file
	def do_gtk_bookmarks( self ):
		try:
			file = open( os.path.join( os.path.expanduser( '~' ), '.gtk-bookmarks' ), 'r' )
			list = file.readlines()
			for i in range( len( list ) ):
				bookmarkInfo = list[i].split()
				bookmarkInfo[0] = string.join( bookmarkInfo[0].split( "%20" ) )
				ButtonLocation = bookmarkInfo[0]
				ButtonCmd = [self.execapp, ButtonLocation]

				if bookmarkInfo[0][:4] == 'file':
					ButtonIcon = "gnome-fs-directory"

				if bookmarkInfo[0][:3] == 'smb' or bookmarkInfo[0][:3] == 'ssh':
					ButtonIcon = "gnome-fs-network"

				if len( bookmarkInfo ) > 2:
					locationName = ''
					for items in bookmarkInfo[1:]:
						locationName = locationName + items + " "
					locationName = [locationName]
					Button1 = easyButton( ButtonIcon, self.iconsize, locationName, 160, -1 )
					Button1.connect( "clicked", self.ButtonClicked, ButtonCmd )
					Button1.show()
					self.wTree.get_widget( "editable_button_holder" ).pack_start( Button1, False, False, 10 )
				else:
					locationName = [bookmarkInfo[0].split( '/' )[-1:][0]]
					if locationName == [""]:
						locationName = "/"
					Button1 = easyButton( ButtonIcon, self.iconsize, locationName, 160, -1 )
					Button1.connect( "clicked", self.ButtonClicked, ButtonCmd )
					Button1.show()
					self.wTree.get_widget( "editable_button_holder" ).pack_start( Button1, False, False, 10 )
			file.close()
		except:
			pass


	#Add standard places
	def do_standard_places( self ):
		
		places = [ "/usr/share/applications/nautilus-computer.desktop", "/usr/share/applications/nautilus-home.desktop", "/usr/share/applications/network-scheme.desktop" ]
		
		for place in places:
			Button1 = ApplicationLauncher( place, self.iconsize )
			Button1.connect( "clicked", self.ButtonClicked, "" )
			Button1.show()
			self.wTree.get_widget( "places_button_holder" ).pack_start( Button1, False, False, 10 )
			self.PlaceTips.set_tip( Button1, Button1.getTooltip() )

		PlaceData = [_("Desktop"), _("Desktop"), "", "gnome-fs-desktop"]
		Button1 = easyButton( PlaceData[3], self.iconsize, [PlaceData[0]], 160, -1 )
		Button1.connect( "clicked", self.ButtonClicked, self.desktop )
		Button1.show()
		self.wTree.get_widget( "places_button_holder" ).pack_start( Button1, False, False, 10 )
		self.PlaceTips.set_tip(Button1, PlaceData[1])

		Button1 = easyButton( "user-trash", self.iconsize, [_("Trash")], 160, -1 )
		Button1.connect( "clicked", self.ButtonClicked, self.trash )
		Button1.show()
		self.wTree.get_widget( "places_button_holder" ).pack_start( Button1, False, False, 10 )
		self.PlaceTips.set_tip(Button1, _("Trash"))  

	#Add mounted volumes
	def do_mounted_volumes( self ):

		import gtop

		Button1 = easyButton( "gnome-dev-harddisk", self.iconsize, ["Filesystem"], 160, -1 )
		Button1.connect( "clicked", self.ButtonClicked, [self.execapp, "/"] )
		self.wTree.get_widget( "places_button_holder" ).pack_start( Button1, False, False, 10 )

		usage = gtop.fsusage("/")
		percent = 100 - (usage.free * 100 / usage.total)
		PlaceUsage = str(percent) + "% used " + str(usage.free * 100 / usage.total) + "% free"
		self.PlaceTips.set_tip(Button1,"/" + "\n" + PlaceUsage)

		for volume in gnomevfs.VolumeMonitor().get_mounted_volumes():
			if ( volume.is_user_visible() and volume.is_mounted() and volume.get_activation_uri() ):
				Button1 = easyButton( volume.get_icon(), self.iconsize, [volume.get_display_name()], 160, -1 )
				Button1.connect( "clicked", self.ButtonClicked, [self.execapp, volume.get_activation_uri().split( "//" )[1]] )
				self.wTree.get_widget( "places_button_holder" ).pack_start( Button1, False, False, 10 )

				PlaceURI = volume.get_activation_uri().split( "//" )[1]
				usage = gtop.fsusage(PlaceURI)
				if (usage.total != 0):
					percent = 100 - (usage.free * 100 / usage.total)
					free = usage.free * 100 / usage.total
				else:
					percent = 0
					free = 0
				PlaceUsage = str(percent) + "% used " + str(free) + "% free"
				self.PlaceTips.set_tip(Button1,PlaceURI + "\n" + PlaceUsage)

	#Add a spacer
	def do_spacer( self ):
		sep1 = gtk.HSeparator()
		Event1 = gtk.EventBox()
		Event1.add( sep1 )
		Event1.show()
		sep1.show()
		self.mintMenuWin.SetPaneColors([Event1])
		#self.itemstocolor.append( Event1 )
		self.wTree.get_widget( "places_button_holder" ).pack_start( Event1, False, False, 10 )

	def ButtonClicked( self, widget, Exec ):
		if Exec:
			Execute( widget, Exec )

				

	#Generate a bookmark entry for the .gtk-bookmarks file
	def generate_bookmark( self, location, description ):
		location = string.join( location.split(), "%20" )
		if description != "":
			location += " " + description
		return location

	def do_plugin( self ):
		   self.do_standard_places()
#		   self.do_mounted_volumes()
#		   self.do_spacer()
#		   self.do_gtk_bookmarks()






#---------------------------------------------------------------------------------------#
# mintMenuconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins mintMenuconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "places.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in mintMenuconfig
		self.heading = "Places"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/mintMenu/plugins/places'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/mintMenu/plugins/places', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_ExeAppEntry_changed" : self.exeappchange,
				"on_DeskCmdEntry_changed" : self.deskchange,
				"on_HomeCmdEntry_changed" : self.homechange,
				"on_NetCmdEntry_changed" : self.netchange,
				"on_CompCmdEntry_changed" : self.compchange,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_PlaceFontSizeSpin_changed" : self.placefontspin,
				"on_PlaceIconSizeSpin_changed" : self.placeiconspin,
				"on_PlaceHSpin_changed" : self.placesh,
				"on_PlaceWSpin_changed" : self.placesw,
				"on_PlaceIconEntry_changed" : self.placeiconchange,
				"on_StickyChkBtn_toggled" : self.placesticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("ExeAppEntry").set_text(SetGconf(self.client, "string", "execute_app", "nautilus"))
		self.desktop_default = string.join( ["nautilus", os.path.join( os.path.expanduser( "~" ), "Desktop" )] )
		self.wTree.get_widget("DeskCmdEntry").set_text(SetGconf(self.client, 'string', 'desktop_cmd', self.desktop_default ))
		self.home_default = string.join( ["nautilus", os.path.expanduser( "~" )] )
		self.wTree.get_widget("HomeCmdEntry").set_text(SetGconf(self.client, 'string', 'home_cmd', self.home_default ))
		self.wTree.get_widget("NetCmdEntry").set_text(SetGconf(self.client, 'string', 'network_cmd', 'nautilus --no-desktop network:' ))
		self.wTree.get_widget("CompCmdEntry").set_text(SetGconf(self.client, 'string', 'computer_cmd', 'nautilus --no-desktop computer:' ))
		self.wTree.get_widget("HideSepChkBtn").set_active(SetGconf(self.client, 'bool', 'hide_separator',False))
		self.wTree.get_widget("PlaceFontSizeSpin").set_value(int(SetGconf(self.client,'int','font_size',9)))
		self.wTree.get_widget("PlaceIconSizeSpin").set_value(int(SetGconf(self.client,'int','icon_size',2)))
		self.wTree.get_widget("PlaceHSpin").set_value(int(SetGconf(self.client,'int','height',280)))
		self.wTree.get_widget("PlaceWSpin").set_value(int(SetGconf(self.client,'int','width',200)))
		self.wTree.get_widget("PlaceIconEntry").set_text(SetGconf(self.client,'string','icon','computer.png'))
		self.wTree.get_widget("StickyChkBtn").set_active(SetGconf(self.client,'bool','sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def exeappchange(self, widget, **kargs):
				self.client.set_string('execute_app',self.wTree.get_widget("ExeAppEntry").get_text())

	def deskchange(self, widget, **kargs):
				self.client.set_string('desktop_cmd',self.wTree.get_widget("DeskCmdEntry").get_text())

	def homechange(self, widget, **kargs):
				self.client.set_string('home_cmd',self.wTree.get_widget("HomeCmdEntry").get_text())

	def netchange(self, widget, **kargs):
				self.client.set_string('network_cmd',self.wTree.get_widget("NetCmdEntry").get_text())

	def compchange(self, widget, **kargs):
				self.client.set_string('computer_cmd',self.wTree.get_widget("CompCmdEntry").get_text())

	def placefontspin(self, widget, **kargs):
		self.client.set_int('font_size',int(self.wTree.get_widget("PlaceFontSizeSpin").get_value()))

	def placeiconspin(self, widget, **kargs):
		self.client.set_int('icon_size',int(self.wTree.get_widget("PlaceIconSizeSpin").get_value()))

	def placesh(self, widget, **kargs):
		self.client.set_int('height',int(self.wTree.get_widget("PlaceHSpin").get_value()))

	def placesw(self, widget, **kargs):
		self.client.set_int('width',int(self.wTree.get_widget("PlaceWSpin").get_value()))

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('hide_separator',True)
		else:
			self.client.set_bool('hide_separator',False)

	def placeiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("PlaceIconEntry").get_text()
		self.client.set_string('icon',self.wTree.get_widget('PlaceIconEntry').get_text())

	def placesticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('sticky',True)
		else:
			self.client.set_bool('sticky',False)
