#!/usr/bin/env python
# mintMenu System Management plugin

import gtk
import gtk.glade
import sys
import os
import gobject
import gconf
import gnomevfs
import string
import gettext

from easyfiles import *
from easybuttons import *
from execute import *
from easygconf import *

gtk.glade.textdomain("mintMenu")
gtk.glade.bindtextdomain("mintMenu", "locale/")
_ = gettext.gettext

class pluginclass:
	def __init__(self, mintMenuWin, toggleButton):

		self.mintMenuWin = mintMenuWin
		self.toggleButton = toggleButton

		# The Glade file for the plugin
		self.gladefile = os.path.join(os.path.dirname(__file__), "system_management.glade")

		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile, "window1")

		# These properties are NECESSARY to maintain consistency
		# throughout mintMenu

		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("window1")
		#dic = {"on_window1_destroy" : gtk.main_quit}
		#self.wTree.signal_autoconnect(dic)

		# Set 'heading' property for plugin
		self.heading = _("System")

		# This should be the first item added to the window in glade
		self.content_holder = self.wTree.get_widget("eventbox1")

		# Items to get custom colors
		self.itemstocolor = [self.wTree.get_widget("eventbox1")]

		# Gconf stuff
		self.gconf = EasyGConf( "/apps/mintMenu/plugins/system-management/" )

		self.width = self.gconf.get( "int", "width", 200)
		self.height = self.gconf.get( "int", "height", 180)
		self.installsoftware = self.gconf.get( "string", "install_software_cmd", "gksu /usr/sbin/synaptic").split()
		self.controlcenter = self.gconf.get( "string", "control_center_cmd","gnome-control-center").split()
		self.terminal = self.gconf.get( "string", "terminal_cmd","gnome-terminal").split()
		self.lockscreen = self.gconf.get( "string", "lock_screen_cmd","gnome-screensaver-command --lock").split()
		self.quit_session = self.gconf.get( "string", "quit_cmd", "gnome-session-save --kill").split()
		self.hideseparator = self.gconf.get( "bool", "hide_separator", False)

		# Allow plugin to be minimized to the left plugin pane
		self.sticky = self.gconf.get( "bool", "sticky",False)
		self.minimized = self.gconf.get( "bool", "minimized", False )

		# Plugin icon
		self.icon = self.gconf.get( "string", "icon", "preferences-system")

	def SetHidden( self, state ):
		if state == True:
			self.gconf.set( "bool", "minimized", True )
		else:
			self.gconf.set( "bool", "minimized", False )


	def ButtonClicked(self,widget, event, Exec):
		self.press_x = event.x
		self.press_y = event.y
		self.Exec = Exec

	def ButtonReleased(self, w, ev):
		if ev.button == 1:
			if not hasattr(self, "press_x") or \
				not w.drag_check_threshold(int(self.press_x),
											 int(self.press_y),
											 int(ev.x),
											 int(ev.y)):

		    		self.mintMenuWin.hide()
				Execute(w,self.Exec)


	def do_setup_window(self):
		self.wTree.get_widget("install_software").connect("button_press_event",self.ButtonClicked,self.installsoftware)
		self.wTree.get_widget("install_software").connect("button_release_event",self.ButtonReleased)
		self.wTree.get_widget("control_center").connect("button_press_event",self.ButtonClicked,self.controlcenter)
		self.wTree.get_widget("control_center").connect("button_release_event",self.ButtonReleased)
		self.wTree.get_widget("terminal").connect("button_press_event",self.ButtonClicked,self.terminal)
		self.wTree.get_widget("terminal").connect("button_release_event",self.ButtonReleased)
		self.wTree.get_widget("lock_screen").connect("button_press_event",self.ButtonClicked,self.lockscreen)
		self.wTree.get_widget("lock_screen").connect("button_release_event",self.ButtonReleased)
		self.wTree.get_widget("quit_session").connect("button_press_event",self.ButtonClicked,self.quit_session)
		self.wTree.get_widget("quit_session").connect("button_release_event",self.ButtonReleased)

	def do_plugin(self):
		self.do_setup_window()






#---------------------------------------------------------------------------------------#
# mintMenuconfig Section Below
#---------------------------------------------------------------------------------------#
# This is the Config Class it must be called "cfgpluginclass'
class cfgpluginclass:
	def __init__(self):
		# The Gladefile for the plugins mintMenuconfig Tab
		self.gladefile = os.path.join(os.path.dirname(__file__), "system_management.glade")
		# Read GLADE file
		self.wTree = gtk.glade.XML(self.gladefile,"config")
		# Set 'window' property for the plugin (Must be the root widget)
		self.window = self.wTree.get_widget("config")
		# Set Heading, this will be used for the tab label in mintMenuconfig
		self.heading = "System"
		# Content Place Holder must be eventbox1
		self.content_holder = self.wTree.get_widget("eventbox2")
		# GConf Stuff - This just makes sure a gconf path is there.
		self.gconf_dir = '/apps/mintMenu/plugins/system-management'
		self.client = gconf.client_get_default()
		self.client.add_dir('/apps/mintMenu/plugins/system-management', gconf.CLIENT_PRELOAD_NONE)
		# Setup the Functions of the Glade files spin controls or entry boxes for Value Changes
		# Tip: For spinbutton controls use the "_value_changed" event not the "_changed" event.
		dic = { "on_window1_destroy" : gtk.main_quit,
				"on_InstCmdEntry_changed" : self.instchange,
				"on_ContCmdEntry_changed" : self.contchange,
				"on_LockCmdEntry_changed" : self.lockchange,
				"on_QuitCmdEntry_changed" : self.quitchange,
				"on_HideSepChkBtn_toggled" : self.hidesep,
				"on_SysFontSizeSpin_changed" : self.sysfontspin,
				"on_SysIconSizeSpin_changed" : self.sysiconspin,
				"on_SysHSpin_changed" : self.sysh,
				"on_SysWSpin_changed" : self.sysw,
				"on_SysIconEntry_changed" : self.sysiconchange,
				"on_StickyChkBtn_toggled" : self.syssticky}
		self.wTree.signal_autoconnect(dic)
		# Read Values from Gconf and Set Default Values if they don't exist.
		self.wTree.get_widget("InstCmdEntry").set_text(self.gconf.get( "string","install_software_cmd", "gksu /usr/sbin/synaptic"))
		self.wTree.get_widget("ContCmdEntry").set_text(self.gconf.get( "string","control_center_cmd","gnome-control-center"))
		self.wTree.get_widget("LockCmdEntry").set_text(self.gconf.get( "string","lock_screen_cmd","gnome-screensaver-command --lock"))
		self.wTree.get_widget("QuitCmdEntry").set_text(self.gconf.get( "string","quit_cmd", "gnome-session-save --kill"))
		self.wTree.get_widget("HideSepChkBtn").set_active(self.gconf.get( 'bool', 'hide_separator',False))
		self.wTree.get_widget("SysFontSizeSpin").set_value(int(self.gconf.get( 'int','font_size',9)))
		self.wTree.get_widget("SysIconSizeSpin").set_value(int(self.gconf.get( 'int','icon_size',2)))
		self.wTree.get_widget("SysHSpin").set_value(int(self.gconf.get( 'int','height',280)))
		self.wTree.get_widget("SysWSpin").set_value(int(self.gconf.get( 'int','width',200)))
		self.wTree.get_widget("SysIconEntry").set_text(self.gconf.get( 'string','icon','computer.png'))
		self.wTree.get_widget("StickyChkBtn").set_active(self.gconf.get( 'bool','sticky',False))

	# Functions for Changing Values when items checked or altered Section
	def instchange(self, widget, **kargs):
		self.client.set_string('install_software_cmd',self.wTree.get_widget("InstCmdEntry").get_text())

	def contchange(self, widget, **kargs):
		self.client.set_string('control_center_cmd',self.wTree.get_widget("ContCmdEntry").get_text())

	def lockchange(self, widget, **kargs):
		self.client.set_string('lock_screen_cmd',self.wTree.get_widget("LockCmdEntry").get_text())

	def quitchange(self, widget, **kargs):
		self.client.set_string('quit_cmd',self.wTree.get_widget("QuitCmdEntry").get_text())

	def sysfontspin(self, widget, **kargs):
		self.client.set_int('font_size',int(self.wTree.get_widget("SysFontSizeSpin").get_value()))

	def sysiconspin(self, widget, **kargs):
		self.client.set_int('icon_size',int(self.wTree.get_widget("SysIconSizeSpin").get_value()))

	def sysh(self, widget, **kargs):
		self.client.set_int('height',int(self.wTree.get_widget("SysHSpin").get_value()))

	def sysw(self, widget, **kargs):
		self.client.set_int('width',int(self.wTree.get_widget("SysWSpin").get_value()))

	def hidesep(self, widget, **kargs):
		if self.wTree.get_widget("HideSepChkBtn").get_active() == True:
			self.client.set_bool('hide_separator',True)
		else:
			self.client.set_bool('hide_separator',False)

	def sysiconchange(self, widget, **kargs):
		self.icon=self.wTree.get_widget("SysIconEntry").get_text()
		self.client.set_string('icon',self.wTree.get_widget('SysIconEntry').get_text())

	def syssticky(self, widget, **kargs):
		if self.wTree.get_widget("StickyChkBtn").get_active() == True:
			self.client.set_bool('sticky',True)
		else:
			self.client.set_bool('sticky',False)

